@@ -32,8 +32,12 @@ class FileProxy(object):
         pass
 
     def write(self, data, *a, **kw):
-        self.f.write(data, *a, **kw)
-        self.f.flush()
+        try:
+            self.f.write(data, *a, **kw)
+            self.f.flush()
+        except socket.error as e:
+            if get_errno(e) != errno.EPIPE:
+                raise
 
     def readline(self, *a):
         return self.f.readline(*a).replace('\r\n', '\n')
@@ -99,14 +103,18 @@ def backdoor_server(sock, locals=None):
 
     print("backdoor server listening on %s" % (listening_on,))
     try:
-        try:
-            while True:
+        while True:
+            socketpair = None
+            try:
                 socketpair = sock.accept()
                 backdoor(socketpair, locals)
-        except socket.error as e:
-            # Broken pipe means it was shutdown
-            if get_errno(e) != errno.EPIPE:
-                raise
+            except socket.error as e:
+                # Broken pipe means it was shutdown
+                if get_errno(e) != errno.EPIPE:
+                    raise
+            finally:
+                if socketpair:
+                    socketpair[0].close()
     finally:
         sock.close()
 
